@@ -40,7 +40,7 @@
 #define _XOPEN_SOURCE	600 /* glibc2 needs this (man strptime) */
 
 #include <libxml/uri.h>
-
+#include <gtk/gtk.h>
 #include <glib.h>
 #include <pango/pango-types.h>
 
@@ -65,8 +65,8 @@ static gchar *lifereaUserPath = NULL;
    mode which can produce invalid UTF-8. 
    
    The valid or a corrected string is returned. The original XML 
-   string is modified (FIXME: not sure if this is good). */
-gchar * common_utf8_fix(xmlChar *string) {
+   string is modified . */
+gchar * common_utf8_fix(gchar *string) {
 	const gchar	*invalid_offset;
 
 	if(NULL == string)
@@ -269,8 +269,8 @@ time_t parseISO8601Date(gchar *date) {
 	return 0;
 }
 
-gchar *dayofweek[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
-gchar *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
+static const gchar *dayofweek[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
+static const gchar *months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 
 gchar *createRFC822Date(const time_t *time) {
 	struct tm *tm;
@@ -280,14 +280,36 @@ gchar *createRFC822Date(const time_t *time) {
 					   months[tm->tm_mon], 1900 + tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec);
 }
 
-/* this table of RFC822 timezones is from gmime-utils.c of the gmime API */
+/* in theory, we'd need only the RFC822 timezones here
+   in practice, feeds also use other timezones...        */
 static struct {
-	char *name;
+	const char *name;
 	int offset;
 } tz_offsets [] = {
-	{ "UT", 0 },
+	{ "IDLW", -1200 },
+	{ "HAST", -1000 },
+	{ "AKST", -900 },
+	{ "AKDT", -800 },
+	{ "WESZ", 100 },
+	{ "WEST", 100 },
+	{ "WEDT", 100 },
+	{ "MEST", 200 },
+	{ "MESZ", 200 },
+	{ "CEST", 200 },
+	{ "CEDT", 200 },
+	{ "EEST", 300 },
+	{ "EEDT", 300 },
+	{ "IRST", 430 },
+	{ "CNST", 800 },
+	{ "ACST", 930 },
+	{ "ACDT", 1030 },
+	{ "AEST", 1000 },
+	{ "AEDT", 1100 },
+	{ "IDLE", 1200 },
+	{ "NZST", 1200 },
+	{ "NZDT", 1300 },
 	{ "GMT", 0 },
-	{ "EST", -500 },        /* these are all US timezones.  bloody yanks */
+	{ "EST", -500 },
 	{ "EDT", -400 },
 	{ "CST", -600 },
 	{ "CDT", -500 },
@@ -295,6 +317,30 @@ static struct {
 	{ "MDT", -600 },
 	{ "PST", -800 },
 	{ "PDT", -700 },
+	{ "HDT", -900 },
+	{ "YST", -900 },
+	{ "YDT", -800 },
+	{ "AST", -400 },
+	{ "ADT", -300 },
+	{ "VST", -430 },
+	{ "NST", -330 },
+	{ "NDT", -230 },
+	{ "WET", 0 },
+	{ "WEZ", 0 },
+	{ "IST", 100 },
+	{ "CET", 100 },
+	{ "MEZ", 100 },
+	{ "EET", 200 },
+	{ "MSK", 300 },
+	{ "MSD", 400 },
+	{ "IRT", 330 },
+	{ "IST", 530 },
+	{ "ICT", 700 },
+	{ "JST", 900 },
+	{ "NFT", 1130 },
+	{ "UT", 0 },
+	{ "PT", -800 },
+	{ "BT", 300 },
 	{ "Z", 0 },
 	{ "A", -100 },
 	{ "M", -1200 },
@@ -305,7 +351,8 @@ static struct {
 static time_t common_parse_rfc822_tz(char *token) {
 	int offset = 0;
 	const char *inptr = token;
-	
+	int num_timezones = sizeof(tz_offsets) / sizeof((tz_offsets)[0]);
+
 	if (*inptr == '+' || *inptr == '-') {
 		offset = atoi (inptr);
 	} else {
@@ -314,9 +361,11 @@ static time_t common_parse_rfc822_tz(char *token) {
 		if (*inptr == '(')
 			inptr++;
 
-		for (t = 0; t < 15; t++)
-			if (!strncmp (inptr, tz_offsets[t].name, strlen (tz_offsets[t].name)))
+		for (t = 0; t < num_timezones; t++)
+			if (!strncmp (inptr, tz_offsets[t].name, strlen (tz_offsets[t].name))) {
 				offset = tz_offsets[t].offset;
+				break;
+			}
 	}
 	
 	return 60 * ((offset / 100) * 60 + (offset % 100));
@@ -384,31 +433,34 @@ time_t parseRFC822Date(gchar *date) {
 	return 0;
 }
 
-static void common_check_dir(gchar *path) {
-
-	if(!g_file_test(path, G_FILE_TEST_IS_DIR)) {
-		if(0 != g_mkdir_with_parents(path, S_IRUSR | S_IWUSR | S_IXUSR)) {
-			g_error(_("Cannot create cache directory \"%s\"!"), path);
+static void
+common_check_dir (gchar *path)
+{
+	if (!g_file_test (path, G_FILE_TEST_IS_DIR)) {
+		if (0 != g_mkdir_with_parents (path, S_IRUSR | S_IWUSR | S_IXUSR)) {
+			g_error (_("Cannot create cache directory \"%s\"!"), path);
 		}
 	}
-	g_free(path);
+	g_free (path);
 }
 
-static void common_init_cache_path(void) {
+static void
+common_init_cache_path (void)
+{
 	gchar *cachePath;
 
-	lifereaUserPath = g_strdup_printf("%s" G_DIR_SEPARATOR_S ".liferea_1.4", g_get_home_dir());
-	cachePath = g_strdup_printf("%s" G_DIR_SEPARATOR_S "cache", lifereaUserPath);
+	lifereaUserPath = g_build_filename (g_get_home_dir(), ".liferea_1.6", NULL);
+	cachePath = g_build_filename (lifereaUserPath, "cache", NULL);
 
-	common_check_dir(g_strdup(lifereaUserPath));
-	common_check_dir(g_strdup(cachePath));
-	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "feeds", cachePath));
-	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "favicons", cachePath));
-	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "plugins", cachePath));
-	common_check_dir(g_strdup_printf("%s" G_DIR_SEPARATOR_S "scripts", cachePath));
+	common_check_dir (g_strdup (lifereaUserPath));
+	common_check_dir (g_strdup (cachePath));
+	common_check_dir (g_build_filename (cachePath, "feeds", NULL));
+	common_check_dir (g_build_filename (cachePath, "favicons", NULL));
+	common_check_dir (g_build_filename (cachePath, "plugins", NULL));
+	common_check_dir (g_build_filename (cachePath, "scripts", NULL));
 
-	g_free(cachePath);
-	/* lifereaUserPath reused globally */
+	g_free (cachePath);
+	/* lifereaUserPath is reused globally */
 	
 	/* ensure reasonable default umask */
 	umask (077);
@@ -435,129 +487,6 @@ gchar * common_create_cache_filename(const gchar *folder, const gchar *filename,
 	return result;
 }
 
-const gchar * common_http_error_to_str(gint httpstatus) {
-	gchar	*tmp = NULL;
-	
-	switch(httpstatus) {
-		case 401:tmp = _("You are unauthorized to download this feed. Please update your username and "
-		                 "password in the feed properties dialog box.");break;
-		case 402:tmp = _("Payment Required");break;
-		case 403:tmp = _("Access Forbidden");break;
-		case 404:tmp = _("Resource Not Found");break;
-		case 405:tmp = _("Method Not Allowed");break;
-		case 406:tmp = _("Not Acceptable");break;
-		case 407:tmp = _("Proxy Authentication Required");break;
-		case 408:tmp = _("Request Time-Out");break;
-		case 410:tmp = _("Gone. Resource doesn't exist. Please unsubscribe!");break;
-	}
-	
-	if(!tmp) {
-		switch(httpstatus / 100) {
-			case 3:tmp = _("Feed not available: Server requested unsupported redirection!");break;
-			case 4:tmp = _("Client Error");break;
-			case 5:tmp = _("Server Error");break;
-			default:tmp = _("(unknown networking error happened)");break;
-		}
-	}
-	
-	return tmp;
-}
-
-const gchar * common_netio_error_to_str(gint netstatus) {
-	gchar	*tmp = NULL;
-	
-	switch(netstatus) {
-		case NET_ERR_URL_INVALID:    tmp = _("URL is invalid"); break;
-		case NET_ERR_PROTO_INVALID:  tmp = _("Unsupported network protocol"); break;
-		case NET_ERR_UNKNOWN:
-		case NET_ERR_CONN_FAILED:
-		case NET_ERR_SOCK_ERR:       tmp = _("Error connecting to remote host"); break;
-		case NET_ERR_HOST_NOT_FOUND: tmp = _("Hostname could not be found"); break;
-		case NET_ERR_CONN_REFUSED:   tmp = _("Network connection was refused by the remote host"); break;
-		case NET_ERR_TIMEOUT:        tmp = _("Remote host did not finish sending data"); break;
-		/* Transfer errors */
-		case NET_ERR_REDIRECT_COUNT_ERR: tmp = _("Too many HTTP redirects were encountered"); break;
-		case NET_ERR_REDIRECT_ERR:
-		case NET_ERR_HTTP_PROTO_ERR: 
-		case NET_ERR_GZIP_ERR:           tmp = _("Remote host sent an invalid response"); break;
-		/* These are handled by HTTP error codes...
-		   case NET_ERR_HTTP_410:
-		   case NET_ERR_HTTP_404:
-		   case NET_ERR_HTTP_NON_200:
-		*/
-		case NET_ERR_AUTH_FAILED:
-		case NET_ERR_AUTH_NO_AUTHINFO: tmp = _("Authentication failed"); break;
-		case NET_ERR_AUTH_GEN_AUTH_ERR:
-		case NET_ERR_AUTH_UNSUPPORTED: tmp = _("Webserver's authentication method incompatible with Liferea"); break;
-	}
-	
-	return tmp;
-}
-
-static gchar * byte_to_hex(unsigned char nr) {
-	gchar *result = NULL;
-
-	result = g_strdup_printf("%%%x%x", nr / 0x10, nr % 0x10);
-	return result;
-}
-
-gchar * common_encode_uri_string(gchar *string) {
-	gchar		*newURIString;
-	gchar		*hex, *tmp = NULL;
-	int		i, j, len, bytes;
-
-	/* the UTF-8 string is casted to ASCII to treat
-	   the characters bytewise and convert non-ASCII
-	   compatible chars to URI hexcodes */
-	newURIString = g_strdup("");
-	len = strlen(string);
-	for(i = 0; i < len; i++) {
-		if(g_ascii_isalnum(string[i]) || strchr("-_.!~*'()", (int)string[i]))
-		   	tmp = g_strdup_printf("%s%c", newURIString, string[i]);
-		else if(string[i] == ' ')
-			tmp = g_strdup_printf("%s%%20", newURIString);
-		else if((unsigned char)string[i] <= 127) {
-			tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex(string[i]));g_free(hex);
-		} else {
-			bytes = 0;
-			if(((unsigned char)string[i] >= 192) && ((unsigned char)string[i] <= 223))
-				bytes = 2;
-			else if(((unsigned char)string[i] > 223) && ((unsigned char)string[i] <= 239))
-				bytes = 3;
-			else if(((unsigned char)string[i] > 239) && ((unsigned char)string[i] <= 247))
-				bytes = 4;
-			else if(((unsigned char)string[i] > 247) && ((unsigned char)string[i] <= 251))
-				bytes = 5;
-			else if(((unsigned char)string[i] > 247) && ((unsigned char)string[i] <= 251))
-				bytes = 6;
-				
-			if(0 != bytes) {
-				if((i + (bytes - 1)) > len) {
-					g_warning(_("Unexpected end of character sequence or corrupt UTF-8 encoding! Some characters were dropped!"));
-					break;
-				}
-
-				for(j=0; j < (bytes - 1); j++) {
-					tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)string[i++]));
-					g_free(hex);
-					g_free(newURIString);
-					newURIString = tmp;
-				}
-				tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)string[i]));
-				g_free(hex);
-			} else {
-				/* sh..! */
-				g_error("Internal error while converting UTF-8 chars to HTTP URI!");
-			}
-		}
-		g_free(newURIString); 
-		newURIString = tmp;
-	}
-	g_free(string);
-
-	return newURIString;
-}
-
 xmlChar * common_uri_escape(const xmlChar *url) {
 	xmlChar	*result;
 
@@ -593,66 +522,6 @@ xmlChar * common_build_url(const gchar *url, const gchar *baseURL) {
 	return absURL;
 }
 
-/* The following three functions are to be used for
-   pixmap handling as provided by glade-2 in support.c */
-
-static GList *pixmaps_directories = NULL;
-
-/* Use this function to set the directory containing installed pixmaps. */
-void
-add_pixmap_directory (const gchar     *directory)
-{
-  pixmaps_directories = g_list_prepend (pixmaps_directories,
-                                        g_strdup (directory));
-}
-
-/* This is an internally used function to find pixmap files. */
-static gchar*
-find_pixmap_file (const gchar     *filename)
-{
-	GList *elem;
-
-	/* We step through each of the pixmaps directory to find it. */
-	elem = pixmaps_directories;
-	while (elem) {
-		gchar *pathname = g_strdup_printf ("%s%s%s", (gchar*)elem->data,
-                                	 G_DIR_SEPARATOR_S, filename);
-		if (g_file_test (pathname, G_FILE_TEST_EXISTS))
-			return pathname;
-		g_free (pathname);
-		elem = elem->next;
-	}
-	return NULL;
-}
-
-/* This is an internally used function to create pixmaps. */
-GdkPixbuf *
-create_pixbuf (const gchar *filename)
-{
-	gchar *pathname = NULL;
-	GdkPixbuf *pixbuf;
-	GError *error = NULL;
-
-	if (!filename || !filename[0])
-		return NULL;
-
-	pathname = find_pixmap_file (filename);
-
-	if (!pathname) {
-		g_warning (_("Couldn't find pixmap file: %s"), filename);
-		return NULL;
-	}
-
-	pixbuf = gdk_pixbuf_new_from_file (pathname, &error);
-	if (!pixbuf) {
-		fprintf (stderr, "Failed to load pixbuf file: %s: %s\n",
-		       pathname, error->message);
-		g_error_free (error);
-	}
-	g_free (pathname);
-	return pixbuf;
-}
-
 const gchar * common_get_direction_mark(gchar *text) {
 	PangoDirection		pango_direction = PANGO_DIRECTION_NEUTRAL;
 	GtkTextDirection	gtk_direction;
@@ -682,6 +551,7 @@ const gchar * common_get_direction_mark(gchar *text) {
 	}
 }
 
+#ifndef HAVE_STRSEP
 /* code taken from glibc-2.2.1/sysdeps/generic/strsep.c */
 char * common_strsep (char **stringp, const char *delim) {
 	char *begin, *end;
@@ -724,6 +594,7 @@ char * common_strsep (char **stringp, const char *delim) {
 		*stringp = NULL;
 	return begin;
 }
+#endif  /*  HAVE_STRSEP  */
 
 /* Taken from gaim 24 June 2004, copyrighted by the gaim developers
    under the GPL, etc.... It was slightly modified to free the passed string */
@@ -825,3 +696,18 @@ char *common_strcasestr (const char *phaystack, const char *pneedle)
  ret0:
 	return 0;
 }
+
+time_t
+common_get_mod_time (const gchar *file)
+{
+	struct stat	attribute;
+	struct tm	tm;
+
+	if (stat (file, &attribute) == 0) {
+		gmtime_r (&(attribute.st_mtime), &tm);
+		return mktime (&tm);
+	} else {
+		/* this is no error as this method is used to check for files */
+		return 0;
+	}
+}
