@@ -30,19 +30,10 @@
 #include "metadata.h"
 
 static sqlite3	*db = NULL;
-static guint	stmtCounter = 0;
-static gboolean	transaction = FALSE;	/**< single transaction lock, prevents DB re-opening during transaction */
-
-/** 
- * To avoid loosing statements on crashes, close+reopen the DB from time to time,
- * THIS IS A WORKAROUND TO AVOID A STRANGE EFFECT OF LOOSING ALL TRANSACTIONS ON EXIT. 
- */
-#define MAX_STATEMENTS_BEFORE_RECONNECT	500
 
 /** value structure of statements hash */ 
 typedef struct statement {
 	sqlite3_stmt	*stmt;	/** the prepared statement */
-	gboolean	write;	/** TRUE if statement modifies DB and should be counted to stmtCounter */
 } statement;
 
 /** hash of all prepared statements */
@@ -70,8 +61,6 @@ db_new_statement (const gchar *name, const gchar *sql)
 	struct statement *statement;
 	
 	statement = g_new0 (struct statement, 1);
-	if (strstr (sql, "INSERT") || strstr (sql, "REPLACE"))
-		statement->write = TRUE;
 	db_prepare_stmt (&statement->stmt, sql);
 	
 	if (!statements)
@@ -85,21 +74,9 @@ db_get_statement (const gchar *name)
 {
 	struct statement *statement;
 
-redo:
 	statement = (struct statement *) g_hash_table_lookup (statements, name);
 	if (!statement)
 		g_error ("Fatal: unknown prepared statement \"%s\" requested!", name);	
-		
-	if (statement->write)
-		stmtCounter++;
-		
-	if (!transaction && (stmtCounter > MAX_STATEMENTS_BEFORE_RECONNECT)) {
-		stmtCounter = 0;
-		debug1 (DEBUG_DB, "DB reconnect after %d DB write actions...\n", MAX_STATEMENTS_BEFORE_RECONNECT); 
-		db_deinit ();
-		db_init (FALSE);
-		goto redo;
-	}
 
 	sqlite3_reset (statement->stmt);
 	return statement->stmt;
@@ -113,7 +90,11 @@ db_exec (const gchar *sql)
 	
 	debug1 (DEBUG_DB, "executing SQL: %s", sql);
 	res = sqlite3_exec (db, sql, NULL, NULL, &err);
-	debug2 (DEBUG_DB, " -> result: %d (%s)", res, err?err:"success");	
+	if (1 >= res) {
+		debug2 (DEBUG_DB, " -> result: %d (%s)", res, err?err:"success");
+	} else {
+		g_warning ("Unexpected status on SQL execution: %d (%s)", res, err?err:"success");
+	}
 	sqlite3_free (err);
 }
 
@@ -173,7 +154,7 @@ db_get_schema_version (void)
 	
 /* opening or creation of database */
 void
-db_init (gboolean initial) 
+db_init (void)
 {
 	gchar		*filename;
 	gint		schemaVersion;
@@ -181,8 +162,6 @@ db_init (gboolean initial)
 		
 	debug_enter ("db_init");
 	
-	stmtCounter = 0;
-
 open:
 	filename = common_create_cache_filename (NULL, "liferea", "db");
 	debug1 (DEBUG_DB, "Opening DB file %s...", filename);
@@ -193,274 +172,280 @@ open:
 	
 	sqlite3_extended_result_codes (db, TRUE);
 	
-	if (initial) {
-		/* create info table/check versioning info */				   
-		schemaVersion = db_get_schema_version ();
-		debug1 (DEBUG_DB, "current DB schema version: %d", schemaVersion);
+	/* create info table/check versioning info */				   
+	schemaVersion = db_get_schema_version ();
+	debug1 (DEBUG_DB, "current DB schema version: %d", schemaVersion);
 
-		if (-1 == schemaVersion) {
-			/* no schema version available -> first installation without tables... */
-			db_set_schema_version (SCHEMA_TARGET_VERSION);
-			schemaVersion = SCHEMA_TARGET_VERSION;	/* nothing exists yet, tables will be created below */
+	if (-1 == schemaVersion) {
+		/* no schema version available -> first installation without tables... */
+		db_set_schema_version (SCHEMA_TARGET_VERSION);
+		schemaVersion = SCHEMA_TARGET_VERSION;	/* nothing exists yet, tables will be created below */
+	}
+
+	if (SCHEMA_TARGET_VERSION < schemaVersion)
+		g_error ("Fatal: The cache database was created by a newer version of Liferea than this one!");
+
+	if (SCHEMA_TARGET_VERSION > schemaVersion) {		
+		/* do table migration */
+		if (db_get_schema_version () < 5)
+			g_error ("This version of Liferea doesn't support migrating from such an old DB file!");
+
+		if (db_get_schema_version () == 5 || db_get_schema_version () == 6) {
+			debug0 (DEBUG_DB, "dropping triggers in preparation of database migration");
+			db_exec ("BEGIN; "
+			         "DROP TRIGGER item_removal; "
+				 "DROP TRIGGER item_insert; "
+				 "END;");
 		}
 
-		if (SCHEMA_TARGET_VERSION < schemaVersion)
-			g_error ("Fatal: The cache database was created by a newer version of Liferea than this one!");
-
-		if (SCHEMA_TARGET_VERSION > schemaVersion) {		
-			/* do table migration */
-			if (db_get_schema_version () < 5)
-				g_error ("This version of Liferea doesn't support migrating from such an old DB file!");
-			
-			if (db_get_schema_version () == 5 || db_get_schema_version () == 6) {
-				debug0 (DEBUG_DB, "dropping triggers in preparation of database migration");
-				db_exec ("BEGIN; "
-				         "DROP TRIGGER item_removal; "
-					 "DROP TRIGGER item_insert; "
-					 "END;");
-			}
-
-			if (db_get_schema_version () == 5) {
+		if (db_get_schema_version () == 5) {
 				/* 1.4.9 -> 1.4.10 adding parent_item_id to itemset relation */
-				debug0 (DEBUG_DB, "migrating from schema version 5 to 6 (this drops all comments)");
-				db_exec ("BEGIN; "
-				         "DELETE FROM itemsets WHERE comment = 1; "
-					 "DELETE FROM items WHERE comment = 1; "
-				         "CREATE TEMPORARY TABLE itemsets_backup(item_id,node_id,read,comment); "
-					 "INSERT INTO itemsets_backup SELECT item_id,node_id,read,comment FROM itemsets; "
-					 "DROP TABLE itemsets; "
-					 "CREATE TABLE itemsets ("
-			        	 "   item_id		INTEGER,"
-					 "   parent_item_id     INTEGER,"
-			        	 "   node_id		TEXT,"
-			        	 "   read		INTEGER,"
-					 "   comment            INTEGER,"
-			        	 "   PRIMARY KEY (item_id, node_id)"
-			        	 "); "
-					 "INSERT INTO itemsets SELECT item_id,0,node_id,read,comment FROM itemsets_backup; "
-					 "DROP TABLE itemsets_backup; "
-					 "REPLACE INTO info (name, value) VALUES ('schemaVersion',6); "
-					 "END;");
-			}
-			
-			if (db_get_schema_version () == 6) {
-				/* 1.4.15 -> 1.4.16 adding parent_node_id to itemset relation */
-				debug0 (DEBUG_DB, "migrating from schema version 6 to 7 (this drops all comments)");
-				db_exec ("BEGIN; "
-				         "DELETE FROM itemsets WHERE comment = 1; "
-					 "DELETE FROM items WHERE comment = 1; "
-				         "CREATE TEMPORARY TABLE itemsets_backup(item_id,node_id,read,comment); "
-					 "INSERT INTO itemsets_backup SELECT item_id,node_id,read,comment FROM itemsets; "
-					 "DROP TABLE itemsets; "
-					 "CREATE TABLE itemsets ("
-			        	 "   item_id		INTEGER,"
-					 "   parent_item_id     INTEGER,"
-			        	 "   node_id		TEXT,"
-					 "   parent_node_id     TEXT,"
-			        	 "   read		INTEGER,"
-					 "   comment            INTEGER,"
-			        	 "   PRIMARY KEY (item_id, node_id)"
-			        	 "); "
-					 "INSERT INTO itemsets SELECT item_id,0,node_id,node_id,read,comment FROM itemsets_backup; "
-					 "DROP TABLE itemsets_backup; "
-					 "REPLACE INTO info (name, value) VALUES ('schemaVersion',7); "
-					 "END;");
-			}
-
-			if (SCHEMA_TARGET_VERSION != db_get_schema_version ())
-				g_error ("Fatal: DB schema migration failed! Running with --debug-db could give some hints!");
-
-			db_deinit ();			
-			debug0 (DEBUG_DB, "Reopening DB after migration...");
-			goto open;
+			debug0 (DEBUG_DB, "migrating from schema version 5 to 6 (this drops all comments)");
+			db_exec ("BEGIN; "
+			         "DELETE FROM itemsets WHERE comment = 1; "
+				 "DELETE FROM items WHERE comment = 1; "
+			         "CREATE TEMPORARY TABLE itemsets_backup(item_id,node_id,read,comment); "
+				 "INSERT INTO itemsets_backup SELECT item_id,node_id,read,comment FROM itemsets; "
+				 "DROP TABLE itemsets; "
+				 "CREATE TABLE itemsets ("
+		        	 "   item_id		INTEGER,"
+				 "   parent_item_id     INTEGER,"
+		        	 "   node_id		TEXT,"
+		        	 "   read		INTEGER,"
+				 "   comment            INTEGER,"
+		        	 "   PRIMARY KEY (item_id, node_id)"
+		        	 "); "
+				 "INSERT INTO itemsets SELECT item_id,0,node_id,read,comment FROM itemsets_backup; "
+				 "DROP TABLE itemsets_backup; "
+				 "REPLACE INTO info (name, value) VALUES ('schemaVersion',6); "
+				 "END;");
 		}
 
-		debug_start_measurement (DEBUG_DB);
-		db_begin_transaction ();
+		if (db_get_schema_version () == 6) {
+			/* 1.4.15 -> 1.4.16 adding parent_node_id to itemset relation */
+			debug0 (DEBUG_DB, "migrating from schema version 6 to 7 (this drops all comments)");
+			db_exec ("BEGIN; "
+			         "DELETE FROM itemsets WHERE comment = 1; "
+				 "DELETE FROM items WHERE comment = 1; "
+			         "CREATE TEMPORARY TABLE itemsets_backup(item_id,node_id,read,comment); "
+				 "INSERT INTO itemsets_backup SELECT item_id,node_id,read,comment FROM itemsets; "
+				 "DROP TABLE itemsets; "
+				 "CREATE TABLE itemsets ("
+		        	 "   item_id		INTEGER,"
+				 "   parent_item_id     INTEGER,"
+		        	 "   node_id		TEXT,"
+				 "   parent_node_id     TEXT,"
+		        	 "   read		INTEGER,"
+				 "   comment            INTEGER,"
+		        	 "   PRIMARY KEY (item_id, node_id)"
+		        	 "); "
+				 "INSERT INTO itemsets SELECT item_id,0,node_id,node_id,read,comment FROM itemsets_backup; "
+				 "DROP TABLE itemsets_backup; "
+				 "REPLACE INTO info (name, value) VALUES ('schemaVersion',7); "
+				 "END;");
+		}
 
-		/* create tables if they do not exist yet */
-		db_exec ("CREATE TABLE items ("
-	        	 "   title		TEXT,"
-	        	 "   read		INTEGER,"
-	        	 "   new		INTEGER,"
-	        	 "   updated		INTEGER,"
-	        	 "   popup		INTEGER,"
-	        	 "   marked		INTEGER,"
-	        	 "   source		TEXT,"
-	        	 "   source_id		TEXT,"
-	        	 "   valid_guid		INTEGER,"
-	        	 "   real_source_url	TEXT,"
-	        	 "   real_source_title	TEXT,"
-	        	 "   description	TEXT,"
-	        	 "   date		INTEGER,"
-	        	 "   comment_feed_id	TEXT,"
-			 "   comment            INTEGER"
-	        	 ");");
+		if (SCHEMA_TARGET_VERSION != db_get_schema_version ())
+			g_error ("Fatal: DB schema migration failed! Running with --debug-db could give some hints!");
 
-		db_exec ("CREATE INDEX items_idx ON items (source_id);");
-		db_exec ("CREATE INDEX items_idx2 ON items (comment_feed_id);");
+		db_deinit ();			
+		debug0 (DEBUG_DB, "Reopening DB after migration...");
+		goto open;
+	}
 
-		db_exec ("CREATE TABLE itemsets ("
-	        	 "   item_id		INTEGER,"
-			 "   parent_item_id     INTEGER,"
-	        	 "   node_id		TEXT,"
-			 "   parent_node_id     TEXT,"
-	        	 "   read		INTEGER,"
-			 "   comment            INTEGER,"
-	        	 "   PRIMARY KEY (item_id, node_id)"
-	        	 ");");
-
-		db_exec ("CREATE INDEX itemset_idx  ON itemsets (node_id);");
-		db_exec ("CREATE INDEX itemset_idx2 ON itemsets (item_id);");
+	/* Schema creation */
 		
-		db_exec ("CREATE TABLE metadata ("
-	        	 "   item_id		INTEGER,"
-	        	 "   nr              	INTEGER,"
-	        	 "   key             	TEXT,"
-	        	 "   value           	TEXT,"
-	        	 "   PRIMARY KEY (item_id, nr)"
-	        	 ");");
+	debug_start_measurement (DEBUG_DB);
+	db_begin_transaction ();
 
-		db_exec ("CREATE INDEX metadata_idx ON metadata (item_id);");
+	/* 1. Create tables if they do not exist yet */
+	db_exec ("CREATE TABLE items ("
+        	 "   title		TEXT,"
+        	 "   read		INTEGER,"
+        	 "   new		INTEGER,"	/* FIXME: drop this deprecated column on next DB migration! */
+        	 "   updated		INTEGER,"
+        	 "   popup		INTEGER,"
+        	 "   marked		INTEGER,"
+        	 "   source		TEXT,"
+        	 "   source_id		TEXT,"
+        	 "   valid_guid		INTEGER,"
+        	 "   real_source_url	TEXT,"		/* FIXME: drop this deprecated column on next DB migration! */
+        	 "   real_source_title	TEXT,"		/* FIXME: drop this deprecated column on next DB migration! */
+        	 "   description	TEXT,"
+        	 "   date		INTEGER,"
+        	 "   comment_feed_id	TEXT,"
+		 "   comment            INTEGER"
+        	 ");");
 
-		db_exec ("CREATE TABLE subscription ("
-	        	 "   node_id            STRING,"
-			 "   source             STRING,"
-			 "   orig_source        STRING,"
-			 "   filter_cmd         STRING,"
-			 "   update_interval	INTEGER,"
-			 "   default_interval   INTEGER,"
-			 "   discontinued       INTEGER,"
-			 "   available          INTEGER,"
-	        	 "   PRIMARY KEY (node_id)"
-			 ");");
+	db_exec ("CREATE INDEX items_idx ON items (source_id);");
+	db_exec ("CREATE INDEX items_idx2 ON items (comment_feed_id);");
 
-		db_exec ("CREATE TABLE update_state ("
-	        	 "   node_id            STRING,"
-			 "   last_modified      STRING,"
-			 "   etag               STRING,"
-			 "   last_update        INTEGER,"
-			 "   last_favicon_update INTEGER,"
-	        	 "   PRIMARY KEY (node_id)"
-			 ");");
+	db_exec ("CREATE TABLE itemsets ("
+        	 "   item_id		INTEGER,"
+		 "   parent_item_id     INTEGER,"
+        	 "   node_id		TEXT,"
+		 "   parent_node_id     TEXT,"
+        	 "   read		INTEGER,"
+		 "   comment            INTEGER,"
+        	 "   PRIMARY KEY (item_id, node_id)"
+        	 ");");
 
-		db_exec ("CREATE TABLE subscription_metadata ("
-	        	 "   node_id            STRING,"
-			 "   nr                 INTEGER,"
-			 "   key                TEXT,"
-			 "   value              TEXT,"
-			 "   PRIMARY KEY (node_id, nr)"
-			 ");");
+	db_exec ("CREATE INDEX itemset_idx  ON itemsets (node_id);");
+	db_exec ("CREATE INDEX itemset_idx2 ON itemsets (item_id);");
+		
+	db_exec ("CREATE TABLE metadata ("
+        	 "   item_id		INTEGER,"
+        	 "   nr              	INTEGER,"
+        	 "   key             	TEXT,"
+        	 "   value           	TEXT,"
+        	 "   PRIMARY KEY (item_id, nr)"
+        	 ");");
 
-		db_exec ("CREATE INDEX subscription_metadata_idx ON subscription_metadata (node_id);");
+	db_exec ("CREATE INDEX metadata_idx ON metadata (item_id);");
+		
+	db_exec ("CREATE TABLE attention_stats ("
+	         "   category_id	TEXT,"
+	         "   category_name	TEXT,"
+		 "   count              INTEGER,"
+		 "   PRIMARY KEY (category_id)"
+		 ");");
 
-		db_exec ("CREATE TABLE node ("
-	        	 "   node_id		STRING,"
-	        	 "   parent_id		STRING,"
-	        	 "   title		STRING,"
-			 "   type		INTEGER,"
-			 "   expanded           INTEGER,"
-			 "   view_mode		INTEGER,"
-			 "   sort_column	INTEGER,"
-			 "   sort_reversed	INTEGER,"
-			 "   PRIMARY KEY (node_id)"
-	        	 ");");
+	db_exec ("CREATE TABLE subscription ("
+        	 "   node_id            STRING,"
+		 "   source             STRING,"
+		 "   orig_source        STRING,"
+		 "   filter_cmd         STRING,"
+		 "   update_interval	INTEGER,"
+		 "   default_interval   INTEGER,"
+		 "   discontinued       INTEGER,"
+		 "   available          INTEGER,"
+        	 "   PRIMARY KEY (node_id)"
+		 ");");
 
-		db_exec ("CREATE TABLE view_state ("
-		         "   node_id            STRING,"
-			 "   unread             INTEGER,"
-			 "   count              INTEGER,"
-			 "   PRIMARY KEY (node_id)"
-			 ");");
+	db_exec ("CREATE TABLE update_state ("
+        	 "   node_id            STRING,"
+		 "   last_modified      STRING,"
+		 "   etag               STRING,"
+		 "   last_update        INTEGER,"
+		 "   last_favicon_update INTEGER,"
+        	 "   PRIMARY KEY (node_id)"
+		 ");");
+
+	db_exec ("CREATE TABLE subscription_metadata ("
+        	 "   node_id            STRING,"
+		 "   nr                 INTEGER,"
+		 "   key                TEXT,"
+		 "   value              TEXT,"
+		 "   PRIMARY KEY (node_id, nr)"
+		 ");");
+
+	db_exec ("CREATE INDEX subscription_metadata_idx ON subscription_metadata (node_id);");
+
+	db_exec ("CREATE TABLE node ("
+        	 "   node_id		STRING,"
+        	 "   parent_id		STRING,"
+        	 "   title		STRING,"
+		 "   type		INTEGER,"
+		 "   expanded           INTEGER,"
+		 "   view_mode		INTEGER,"
+		 "   sort_column	INTEGER,"
+		 "   sort_reversed	INTEGER,"
+		 "   PRIMARY KEY (node_id)"
+        	 ");");
+
+	db_exec ("CREATE TABLE view_state ("
+	         "   node_id            STRING,"
+		 "   unread             INTEGER,"
+		 "   count              INTEGER,"
+		 "   PRIMARY KEY (node_id)"
+		 ");");
+
+	db_end_transaction ();
+	debug_end_measurement (DEBUG_DB, "table setup");
+		
+	/* 2. Removing old triggers */
+	db_exec ("DROP TRIGGER item_insert;");
+	db_exec ("DROP TRIGGER item_update;");
+	db_exec ("DROP TRIGGER item_removal;");
+	db_exec ("DROP TRIGGER subscription_removal;");
+		
+	/* 3. Cleanup of DB */
+
+	{
+		gchar *sql;
+		sqlite3_stmt *stmt;
+		
+		debug0 (DEBUG_DB, "Checking for items not referenced in table 'itemsets'...");
+		db_exec ("BEGIN; "
+		         "   CREATE TEMP TABLE tmp_id ( id );"
+			 "   INSERT INTO tmp_id SELECT ROWID FROM items WHERE ROWID NOT IN (SELECT item_id FROM itemsets);"
+			 "   DELETE FROM items WHERE ROWID IN (SELECT id FROM tmp_id LIMIT 1000);"
+			 "   DROP TABLE tmp_id;"
+			 "END;");
 			 
-		db_end_transaction ();
-		debug_end_measurement (DEBUG_DB, "table setup");
+		debug0 (DEBUG_DB, "Checking for invalid item ids in table 'itemsets'...");
+		db_exec ("BEGIN; "
+		         "   CREATE TEMP TABLE tmp_id ( id );"
+		         "   INSERT INTO tmp_id SELECT item_id FROM itemsets WHERE item_id NOT IN (SELECT ROWID FROM items);"
+		         /* limit to 1000 items as it is very slow */
+		         "   DELETE FROM itemsets WHERE item_id IN (SELECT id FROM tmp_id LIMIT 1000);"
+		         "   DROP TABLE tmp_id;"
+			 "END;");
 
-		/* 2. Removing old triggers */
-		db_exec ("DROP TRIGGER item_insert;");
-		db_exec ("DROP TRIGGER item_update;");
-		db_exec ("DROP TRIGGER item_removal;");
-		db_exec ("DROP TRIGGER subscription_removal;");
-		
-		/* 3. Cleanup of DB */
-
-		if (initial) {
-			gchar *sql;
-			sqlite3_stmt *stmt;
-			
-			debug0 (DEBUG_DB, "Checking for items not referenced in table 'itemsets'...");
-			db_exec ("BEGIN; "
-			         "   CREATE TEMP TABLE tmp_id ( id );"
-				 "   INSERT INTO tmp_id SELECT ROWID FROM items WHERE ROWID NOT IN (SELECT item_id FROM itemsets);"
-				 "   DELETE FROM items WHERE ROWID IN (SELECT id FROM tmp_id LIMIT 1000);"
-				 "   DROP TABLE tmp_id;"
-				 "END;");
+		/* Note: do not check on subscriptions here, as non-subscription node
+		   types (e.g. news bin) do contain items too. */
+		debug0 (DEBUG_DB, "Checking for items without a feed list node...\n");
+		db_exec ("DELETE FROM itemsets WHERE comment = 0 AND node_id NOT IN "
+	        	 "(SELECT node_id FROM node);");
 				 
-			debug0 (DEBUG_DB, "Checking for invalid item ids in table 'itemsets'...");
-			db_exec ("BEGIN; "
-			         "   CREATE TEMP TABLE tmp_id ( id );"
-			         "   INSERT INTO tmp_id SELECT item_id FROM itemsets WHERE item_id NOT IN (SELECT ROWID FROM items);"
-			         /* limit to 1000 items as it is very slow */
-			         "   DELETE FROM itemsets WHERE item_id IN (SELECT id FROM tmp_id LIMIT 1000);"
-			         "   DROP TABLE tmp_id;"
-				 "END;");
-
-			/* Note: do not check on subscriptions here, as non-subscription node
-			   types (e.g. news bin) do contain items too. */
-			debug0 (DEBUG_DB, "Checking for items without a feed list node...");
-			db_exec ("DELETE FROM itemsets WHERE comment = 0 AND node_id NOT IN "
-		        	 "(SELECT node_id FROM node);");
-				 
-			debug0 (DEBUG_DB, "Checking for stale views not listed in feed list.");
-			sql = sqlite3_mprintf("SELECT name FROM sqlite_master WHERE type='view' AND name not in ("
+		debug0 (DEBUG_DB, "Checking for stale views not listed in feed list.");
+		sql = sqlite3_mprintf("SELECT name FROM sqlite_master WHERE type='view' AND name not in ("
 			                      "SELECT \"view_\"||node_id FROM node WHERE type='vfolder');");
-			res = sqlite3_prepare_v2 (db, sql, -1, &stmt, NULL);
-			sqlite3_free (sql);
-			if (SQLITE_OK != res) {
-				debug1 (DEBUG_DB, "Could not check for stale views (error=%d)", res);
-			} else {
-				sqlite3_reset (stmt);
+		res = sqlite3_prepare_v2 (db, sql, -1, &stmt, NULL);
+		sqlite3_free (sql);
+		if (SQLITE_OK != res) {
+			debug1 (DEBUG_DB, "Could not check for stale views (error=%d)", res);
+		} else {
+			sqlite3_reset (stmt);
 
 				while (sqlite3_step (stmt) == SQLITE_ROW)
-					db_view_remove (sqlite3_column_text (stmt, 0) + strlen("view_"));
+				db_view_remove (sqlite3_column_text (stmt, 0) + strlen("view_"));
 				
-				sqlite3_finalize (stmt);
-			}
-	 
-			debug0 (DEBUG_DB, "DB cleanup finished. Continuing startup.");
+			sqlite3_finalize (stmt);
 		}
-		
-		/* 4. Creating triggers (after cleanup so it is not slowed down by triggers) */
-
-		db_exec ("CREATE TRIGGER item_insert INSERT ON items "
-	        	 "BEGIN "
-	        	 "   UPDATE itemsets SET read = new.read "
-	        	 "   WHERE item_id = new.ROWID; "
-	        	 "END;");
-
-		db_exec ("CREATE TRIGGER item_update UPDATE ON items "
-	        	 "BEGIN "
-	        	 "   UPDATE itemsets SET read = new.read "
-	        	 "   WHERE item_id = new.ROWID; "
-	        	 "END;");
-
-		/* This trigger does explicitely not remove comments! */
-		db_exec ("CREATE TRIGGER item_removal DELETE ON itemsets "
-	        	 "BEGIN "
-	        	 "   DELETE FROM items WHERE ROWID = old.item_id; "
-			 "   DELETE FROM metadata WHERE item_id = old.item_id; "
-	        	 "END;");
-		
-		db_exec ("CREATE TRIGGER subscription_removal DELETE ON subscription "
-	        	 "BEGIN "
-			 "   DELETE FROM node WHERE node_id = old.node_id; "
-	        	 "   DELETE FROM update_state WHERE node_id = old.node_id; "
-			 "   DELETE FROM subscription_metadata WHERE node_id = old.node_id; "
-	        	 "END;");
-
-		/* Note: view counting triggers are set up in the view preparation code (see db_view_create()) */		
+	 
+		debug0 (DEBUG_DB, "DB cleanup finished. Continuing startup.");
 	}
-	
+		
+	/* 4. Creating triggers (after cleanup so it is not slowed down by triggers) */
+
+	db_exec ("CREATE TRIGGER item_insert INSERT ON items "
+        	 "BEGIN "
+        	 "   UPDATE itemsets SET read = new.read "
+        	 "   WHERE item_id = new.ROWID; "
+        	 "END;");
+
+	db_exec ("CREATE TRIGGER item_update UPDATE ON items "
+        	 "BEGIN "
+        	 "   UPDATE itemsets SET read = new.read "
+        	 "   WHERE item_id = new.ROWID; "
+        	 "END;");
+
+	/* This trigger does explicitely not remove comments! */
+	db_exec ("CREATE TRIGGER item_removal DELETE ON itemsets "
+        	 "BEGIN "
+        	 "   DELETE FROM items WHERE ROWID = old.item_id; "
+		 "   DELETE FROM metadata WHERE item_id = old.item_id; "
+        	 "END;");
+		
+	db_exec ("CREATE TRIGGER subscription_removal DELETE ON subscription "
+        	 "BEGIN "
+		 "   DELETE FROM node WHERE node_id = old.node_id; "
+        	 "   DELETE FROM update_state WHERE node_id = old.node_id; "
+		 "   DELETE FROM subscription_metadata WHERE node_id = old.node_id; "
+        	 "END;");
+
+	/* Note: view counting triggers are set up in the view preparation code (see db_view_create()) */		
 	/* prepare statements */
 	
 	db_new_statement ("itemsetLoadStmt",
@@ -490,14 +475,6 @@ open:
 	db_new_statement ("itemsetRemoveAllStmt",
 	                  "DELETE FROM itemsets WHERE parent_node_id = ?");
 
-	db_new_statement ("itemsetMarkAllUpdatedStmt",
-	                  "UPDATE items SET updated = 0 WHERE ROWID IN "
-			  "(SELECT item_id FROM itemsets WHERE node_id = ?)");
-			
-	db_new_statement ("itemsetMarkAllOldStmt",
-	                  "UPDATE items SET new = 0 WHERE ROWID IN "
-			  "(SELECT item_id FROM itemsets WHERE node_id = ?)");
-
 	db_new_statement ("itemsetMarkAllPopupStmt",
 	                  "UPDATE items SET popup = 0 WHERE ROWID IN "
 			  "(SELECT item_id FROM itemsets WHERE node_id = ?)");		
@@ -513,8 +490,6 @@ open:
 	                  "items.source,"
 	                  "items.source_id,"
 	                  "items.valid_guid,"
-	                  "items.real_source_url,"
-	                  "items.real_source_title,"
 	                  "items.description,"
 	                  "items.date,"
 		          "items.comment_feed_id,"
@@ -538,18 +513,17 @@ open:
 	                  "source,"
 	                  "source_id,"
 	                  "valid_guid,"
-	                  "real_source_url,"
-	                  "real_source_title,"
 	                  "description,"
 	                  "date,"
 		          "comment_feed_id,"
 		          "comment,"
 	                  "ROWID"
-	                  ") values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
+	                  ") values (?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
 			
-	db_new_statement ("itemMarkReadStmt",
-	                  "UPDATE items SET read = 1 WHERE ROWID = ?");
-					
+	db_new_statement ("itemStateUpdateStmt",
+			  "UPDATE items SET read=?, marked=?, updated=? "
+			  "WHERE ROWID=?");
+
 	db_new_statement ("duplicatesFindStmt",
 	                  "SELECT ROWID FROM items WHERE source_id = ?");
 			 
@@ -558,7 +532,7 @@ open:
 			  "(SELECT items.ROWID FROM items WHERE items.source_id = ?)");
 		       
 	db_new_statement ("duplicatesMarkReadStmt",
- 	                  "UPDATE items SET read = 1 WHERE source_id = ?");
+ 	                  "UPDATE items SET read = 1, updated = 0 WHERE source_id = ?");
 						
 	db_new_statement ("metadataLoadStmt",
 	                  "SELECT key,value,nr FROM metadata WHERE item_id = ? ORDER BY nr");
@@ -595,9 +569,6 @@ open:
 	db_new_statement ("subscriptionRemoveStmt",
 	                  "DELETE FROM subscription WHERE node_id = ?");
 			 
-	db_new_statement ("subscriptionListLoadStmt",
-	                  "SELECT node_id FROM subscription");
-			 
 	db_new_statement ("subscriptionLoadStmt",
 	                  "SELECT "
 			  "node_id,"
@@ -619,6 +590,12 @@ open:
 	db_new_statement ("nodeUpdateStmt",
 	                  "REPLACE INTO node (node_id,parent_id,title,type,expanded,view_mode,sort_column,sort_reversed) VALUES (?,?,?,?,?,?,?,?)");
 			  
+	db_new_statement ("attentionStatsLoadStmt",
+	                  "SELECT category_id,category_name,count FROM attention_stats");
+			  
+	db_new_statement ("attentionStatUpdateStmt",
+	                  "REPLACE INTO attention_stats (category_id,category_name,count) VALUES (?,?,?)");
+
 	g_assert (sqlite3_get_autocommit (db));
 	
 	debug_exit ("db_init");
@@ -709,28 +686,34 @@ db_item_metadata_update(itemPtr item)
 static itemPtr
 db_load_item_from_columns (sqlite3_stmt *stmt) 
 {
+	const gchar	*tmp;
+
 	itemPtr item = item_new ();
 	
 	item->readStatus	= sqlite3_column_int (stmt, 1)?TRUE:FALSE;
-	item->newStatus		= sqlite3_column_int (stmt, 2)?TRUE:FALSE;
+	/* newStatus isn't used anymore, FIXME: drop column on migration */
 	item->updateStatus	= sqlite3_column_int (stmt, 3)?TRUE:FALSE;
 	item->popupStatus	= sqlite3_column_int (stmt, 4)?TRUE:FALSE;
 	item->flagStatus	= sqlite3_column_int (stmt, 5)?TRUE:FALSE;
 	item->validGuid		= sqlite3_column_int (stmt, 8)?TRUE:FALSE;
-	item->time		= sqlite3_column_int (stmt, 12);
-	item->commentFeedId	= g_strdup (sqlite3_column_text (stmt, 13));
-	item->isComment		= sqlite3_column_int (stmt, 14);
-	item->id		= sqlite3_column_int (stmt, 15);
-	item->parentItemId	= sqlite3_column_int (stmt, 16);
-	item->nodeId		= g_strdup (sqlite3_column_text (stmt, 17));
-	item->parentNodeId	= g_strdup (sqlite3_column_text (stmt, 18));
+	item->time		= sqlite3_column_int (stmt, 10);
+	item->commentFeedId	= g_strdup (sqlite3_column_text (stmt, 11));
+	item->isComment		= sqlite3_column_int (stmt, 12);
+	item->id		= sqlite3_column_int (stmt, 13);
+	item->parentItemId	= sqlite3_column_int (stmt, 14);
+	item->nodeId		= g_strdup (sqlite3_column_text (stmt, 15));
+	item->parentNodeId	= g_strdup (sqlite3_column_text (stmt, 16));
 
-	item_set_title			(item, sqlite3_column_text(stmt, 0));
-	item_set_source			(item, sqlite3_column_text(stmt, 6));
-	item_set_id			(item, sqlite3_column_text(stmt, 7));	
-	item_set_real_source_url	(item, sqlite3_column_text(stmt, 9));
-	item_set_real_source_title	(item, sqlite3_column_text(stmt, 10));
-	item_set_description		(item, sqlite3_column_text(stmt, 11));
+	item->title		= g_strdup (sqlite3_column_text(stmt, 0));
+	item->sourceId		= g_strdup (sqlite3_column_text(stmt, 7));
+	
+	tmp = sqlite3_column_text(stmt, 6);
+	if (tmp)
+		item->source = g_strdup (tmp);
+		
+	tmp = sqlite3_column_text(stmt, 9);
+	if (tmp)
+		item->description = g_strdup (tmp);
 
 	item->metadata = db_item_metadata_load (item);
 
@@ -843,6 +826,8 @@ db_item_update (itemPtr item)
 	debug3 (DEBUG_DB, "update of item \"%s\" (id=%lu, thread=%p)", item->title, item->id, g_thread_self());
 	debug_start_measurement (DEBUG_DB);
 	
+	db_begin_transaction ();
+
 	if (!item->id) {
 		db_item_set_id (item);
 
@@ -866,20 +851,18 @@ db_item_update (itemPtr item)
 	stmt = db_get_statement ("itemUpdateStmt");
 	sqlite3_bind_text (stmt, 1,  item->title, -1, SQLITE_TRANSIENT);
 	sqlite3_bind_int  (stmt, 2,  item->readStatus?1:0);
-	sqlite3_bind_int  (stmt, 3,  item->newStatus?1:0);
+	/* FIXME: newStatus is not used anymore, drop column on next migration */
 	sqlite3_bind_int  (stmt, 4,  item->updateStatus?1:0);
 	sqlite3_bind_int  (stmt, 5,  item->popupStatus?1:0);
 	sqlite3_bind_int  (stmt, 6,  item->flagStatus?1:0);
 	sqlite3_bind_text (stmt, 7,  item->source, -1, SQLITE_TRANSIENT);
 	sqlite3_bind_text (stmt, 8,  item->sourceId, -1, SQLITE_TRANSIENT);
 	sqlite3_bind_int  (stmt, 9,  item->validGuid?1:0);
-	sqlite3_bind_text (stmt, 10, item->real_source_url, -1, SQLITE_TRANSIENT);
-	sqlite3_bind_text (stmt, 11, item->real_source_title, -1, SQLITE_TRANSIENT);
-	sqlite3_bind_text (stmt, 12, item->description, -1, SQLITE_TRANSIENT);
-	sqlite3_bind_int  (stmt, 13, item->time);
-	sqlite3_bind_text (stmt, 14, item->commentFeedId, -1, SQLITE_TRANSIENT);
-	sqlite3_bind_int  (stmt, 15, item->isComment?1:0);
-	sqlite3_bind_int  (stmt, 16, item->id);
+	sqlite3_bind_text (stmt, 10, item->description, -1, SQLITE_TRANSIENT);
+	sqlite3_bind_int  (stmt, 11, item->time);
+	sqlite3_bind_text (stmt, 12, item->commentFeedId, -1, SQLITE_TRANSIENT);
+	sqlite3_bind_int  (stmt, 13, item->isComment?1:0);
+	sqlite3_bind_int  (stmt, 14, item->id);
 	res = sqlite3_step (stmt);
 
 	if (SQLITE_DONE != res) 
@@ -887,9 +870,35 @@ db_item_update (itemPtr item)
 	
 	db_item_metadata_update (item);
 	
+	db_end_transaction ();
+
 	debug_end_measurement (DEBUG_DB, "item update");
 }
 
+void
+db_item_state_update (itemPtr item)
+{
+	sqlite3_stmt	*stmt;
+	
+	if (!item->id) {
+		db_item_update (item);
+		return;
+	}
+
+	debug_start_measurement (DEBUG_DB);
+	
+	stmt = db_get_statement ("itemStateUpdateStmt");
+	sqlite3_bind_int (stmt, 1, item->readStatus?1:0);
+	sqlite3_bind_int (stmt, 2, item->flagStatus?1:0);
+	sqlite3_bind_int (stmt, 3, item->updateStatus?1:0);
+	sqlite3_bind_int (stmt, 4, item->id);
+
+	if (sqlite3_step (stmt) != SQLITE_DONE) 
+		g_warning ("item state update failed (%s)", sqlite3_errmsg (db));
+	debug_end_measurement (DEBUG_DB, "item state update");
+	
+}
+
 void
 db_item_remove (gulong id) 
 {
@@ -900,7 +909,6 @@ db_item_remove (gulong id)
 	
 	stmt = db_get_statement ("itemsetRemoveStmt");
 	sqlite3_bind_int (stmt, 1, id);
-	sqlite3_bind_int (stmt, 2, id);
 	res = sqlite3_step (stmt);
 
 	if (SQLITE_DONE != res)
@@ -973,70 +981,6 @@ db_itemset_remove_all (const gchar *id)
 		g_warning ("removing all items failed (error code=%d, %s)", res, sqlite3_errmsg (db));
 }
 
-void
-db_item_mark_read (itemPtr item) 
-{
-	sqlite3_stmt	*stmt;
-	gint		res;
-	
-	item->readStatus = TRUE;
-	
-	if (!item->validGuid)
-	{
-		debug1 (DEBUG_DB, "marking item with id=%lu read", item->id);
-			
-		stmt = db_get_statement ("itemMarkReadStmt");
-		sqlite3_bind_int (stmt, 1, item->id);
-		res = sqlite3_step (stmt);
-
-		if (SQLITE_DONE != res)
-			g_warning ("marking item read failed (error code=%d, %s)", res, sqlite3_errmsg (db));
-	}
-	else
-	{
-		debug1 (DEBUG_DB, "marking all duplicates with source id=%s read", item->sourceId);
-		
-		stmt = db_get_statement ("duplicatesMarkReadStmt");
-		sqlite3_bind_text (stmt, 1, item->sourceId, -1, SQLITE_TRANSIENT);
-		res = sqlite3_step (stmt);
-
-		if (SQLITE_DONE != res)
-			g_warning ("marking duplicates read failed (error code=%d, %s)", res, sqlite3_errmsg (db));
-	}
-}
-
-void 
-db_itemset_mark_all_updated (const gchar *id) 
-{
-	sqlite3_stmt	*stmt;
-	gint		res;
-	
-	debug1 (DEBUG_DB, "marking all items updared for item set with %s", id);
-		
-	stmt = db_get_statement ("itemsetMarkAllUpdatedStmt");
-	sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
-	res = sqlite3_step (stmt);
-
-	if (SQLITE_DONE != res)
-		g_warning ("marking all items updated failed (error code=%d, %s)", res, sqlite3_errmsg (db));
-}
-
-void 
-db_itemset_mark_all_old (const gchar *id) 
-{
-	sqlite3_stmt	*stmt;
-	gint		res;
-	
-	debug1 (DEBUG_DB, "marking all items old for item set with %s", id);
-		
-	stmt = db_get_statement ("itemsetMarkAllOldStmt");
-	sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
-	res = sqlite3_step (stmt);
-
-	if (SQLITE_DONE != res)
-		g_warning ("marking all items old failed (error code=%d, %s)", res, sqlite3_errmsg(db));
-}
-
 void 
 db_itemset_mark_all_popup (const gchar *id) 
 {
@@ -1111,8 +1055,6 @@ db_begin_transaction (void)
 	res = sqlite3_exec (db, sql, NULL, NULL, &err);
 	if (SQLITE_OK != res) 
 		g_warning ("Transaction begin failed (%s) SQL: %s", err, sql);
-	else
-		transaction = TRUE;
 	sqlite3_free (sql);
 	sqlite3_free (err);
 }
@@ -1127,8 +1069,6 @@ db_end_transaction (void)
 	res = sqlite3_exec (db, sql, NULL, NULL, &err);
 	if (SQLITE_OK != res) 
 		g_warning ("Transaction end failed (%s) SQL: %s", err, sql);
-	else
-		transaction = FALSE;
 	sqlite3_free (sql);
 	sqlite3_free (err);
 }
@@ -1263,7 +1203,7 @@ db_query_to_sql (guint id, const queryPtr query)
 	}
 			
 	if (itemMatch)
-		sql = sqlite3_mprintf ("SELECT %s %s %s WHERE (%s AND %s)",
+		sql = sqlite3_mprintf ("SELECT %s %s %s WHERE ((%s) AND (%s))",
 		                       columns, from, join, itemMatch, query->conditions);
 	else
 		sql = sqlite3_mprintf ("SELECT %s %s %s WHERE (%s)",
@@ -1498,11 +1438,12 @@ db_view_create (const gchar *id, queryPtr query)
 		res = sqlite3_exec (db, sql, NULL, NULL, &err);
 		if (SQLITE_OK != res) 
 			debug2 (DEBUG_DB, "Create view failed (%s) SQL: %s", err, sql);
-		sqlite3_free (sql);
 		sqlite3_free (err);
 	} else {
 		debug1 (DEBUG_DB, "No need to create view %s as it already exists.", id);
 	}
+
+	sqlite3_free (sql);
 	
 	/* Unconditionally recreate the view specific triggers */
 	db_view_remove_triggers (id);
@@ -1691,9 +1632,6 @@ db_update_state_load (const gchar *id,
 	sqlite3_stmt	*stmt;
 	gint		res;
 	
-	g_assert (NULL == updateState->lastModified);
-	g_assert (NULL == updateState->etag);
-
 	debug2 (DEBUG_DB, "loading subscription %s update state (thread=%p)", id, g_thread_self ());
 	debug_start_measurement (DEBUG_DB);	
 
@@ -1702,8 +1640,7 @@ db_update_state_load (const gchar *id,
 
 	res = sqlite3_step (stmt);
 	if (SQLITE_ROW == res) {
-		updateState->lastModified		= g_strdup (sqlite3_column_text (stmt, 0));
-		updateState->etag			= g_strdup (sqlite3_column_text (stmt, 1));
+		// FIXME: retrieve last modified
 		updateState->lastPoll.tv_sec		= sqlite3_column_int (stmt, 2);
 		updateState->lastFaviconPoll.tv_sec	= sqlite3_column_int (stmt, 3);
 	} else {
@@ -1728,8 +1665,9 @@ db_update_state_save (const gchar *id,
 	stmt = db_get_statement ("updateStateSaveStmt");
 	sqlite3_bind_text (stmt, 1, id, -1, SQLITE_TRANSIENT);
 
-	sqlite3_bind_text (stmt, 2, updateState->lastModified, -1, SQLITE_TRANSIENT);
-	sqlite3_bind_text (stmt, 3, updateState->etag, -1, SQLITE_TRANSIENT);
+	// FIXME: drop etag from DB and make lastModified an INTEGER
+	sqlite3_bind_text (stmt, 2, "", -1, SQLITE_TRANSIENT);
+	sqlite3_bind_text (stmt, 3, "", -1, SQLITE_TRANSIENT);
 	sqlite3_bind_int  (stmt, 4, updateState->lastPoll.tv_sec);
 	sqlite3_bind_int  (stmt, 5, updateState->lastFaviconPoll.tv_sec);
 
@@ -1841,20 +1779,6 @@ db_subscription_remove (const gchar *id)
 	debug_end_measurement (DEBUG_DB, "subscription remove");
 }
 
-GSList *
-db_subscription_list_load (void)
-{
-	sqlite3_stmt	*stmt;
-	GSList		*list = NULL;
-
-	stmt = db_get_statement ("subscriptionListLoadStmt");
-	while (sqlite3_step (stmt) == SQLITE_ROW) {
-		list = g_slist_append(list, g_strdup (sqlite3_column_text (stmt, 0)));
-	}
-
-	return list;
-}
-
 void
 db_node_update (nodePtr node)
 {
@@ -1879,5 +1803,51 @@ db_node_update (nodePtr node)
 		g_warning ("Could not update subscription info %s in DB (error code %d)!", node->id, res);
 		
 	debug_end_measurement (DEBUG_DB, "subscription_update");
+}
+
+void
+db_attention_stat_save (categoryStatPtr stat)
+{
+	sqlite3_stmt	*stmt;
+	gint		res;
 	
+	debug_start_measurement (DEBUG_DB);
+	
+	stmt = db_get_statement ("attentionStatUpdateStmt");
+	sqlite3_bind_text (stmt, 1, stat->id, -1, SQLITE_TRANSIENT);
+	sqlite3_bind_text (stmt, 2, stat->name, -1, SQLITE_TRANSIENT);
+	sqlite3_bind_int  (stmt, 3, stat->count);
+	
+	res = sqlite3_step (stmt);
+	if (SQLITE_DONE != res)
+		g_warning ("Could not save attention statistic data for category %s in DB (error code %d: %s)!", stat->name, res, sqlite3_errmsg (db));
+	
+	debug_end_measurement (DEBUG_DB, "attention_stat_save");
+}
+
+void
+db_attention_stats_load (GHashTable **statHash, GSList **statList)
+{
+	sqlite3_stmt	*stmt;
+	
+	debug_start_measurement (DEBUG_DB);
+
+	*statHash = g_hash_table_new (g_str_hash, g_str_equal);
+	stmt = db_get_statement ("attentionStatsLoadStmt");
+
+	while (sqlite3_step (stmt) == SQLITE_ROW) {
+		categoryStatPtr stat = g_new0 (struct categoryStat, 1);
+		stat->id = g_strdup (sqlite3_column_text (stmt, 0));
+		stat->name = g_strdup (sqlite3_column_text (stmt, 1));
+		stat->count = sqlite3_column_int (stmt, 2);
+		g_hash_table_insert (*statHash, stat->id, (gpointer)stat);
+		*statList = g_slist_append (*statList, (gpointer)stat);
+
+		debug3 (DEBUG_DB, "attention stat for %s (%s) = %d\n",
+		                  sqlite3_column_text(stmt, 0),
+		                  sqlite3_column_text(stmt, 1),
+		                  sqlite3_column_int(stmt, 2));
+	}
+	
+	debug_end_measurement (DEBUG_DB, "attention_stats_load");
 }
