@@ -1,7 +1,7 @@
 /**
- * @file feedlist.h feedlist handling
+ * @file feedlist.h  subscriptions as an hierarchic tree
  *
- * Copyright (C) 2005-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2005-2008 Lars Lindner <lars.lindner@gmail.com>
  *	      
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,49 +21,44 @@
 #ifndef _FEEDLIST_H
 #define _FEEDLIST_H
 
-#include <gtk/gtk.h>
+#include <glib-object.h>
+#include <glib.h>
+
 #include "node.h"
 
-/* feed list icons */
-enum allicons {
-	ICON_READ,
-	ICON_UNREAD,
-	ICON_FLAG,
-	ICON_AVAILABLE,
-	ICON_AVAILABLE_OFFLINE,
-	ICON_UNAVAILABLE,
-	ICON_DEFAULT,
-	ICON_OCS,
-	ICON_FOLDER,
-	ICON_VFOLDER,
-	ICON_NEWSBIN,
-	ICON_EMPTY,
-	ICON_EMPTY_OFFLINE,
-	ICON_ONLINE,
-	ICON_OFFLINE,
-	ICON_UPDATED,
-	ICON_ENCLOSURE,
-	MAX_ICONS
+G_BEGIN_DECLS
+
+#define FEEDLIST_TYPE		(feedlist_get_type ())
+#define FEEDLIST(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), FEEDLIST_TYPE, FeedList))
+#define FEEDLIST_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), FEEDLIST_TYPE, FeedListClass))
+#define IS_FEEDLIST(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), FEEDLIST_TYPE))
+#define IS_FEEDLIST_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), FEEDLIST_TYPE))
+
+typedef struct FeedList		FeedList;
+typedef struct FeedListClass	FeedListClass;
+typedef struct FeedListPrivate	FeedListPrivate;
+
+struct FeedList
+{
+	GObject		parent;
+	
+	/*< private >*/
+	FeedListPrivate	*priv;
 };
 
-extern GdkPixbuf *icons[MAX_ICONS];
+struct FeedListClass 
+{
+	GObjectClass parent_class;	
+};
 
-/** 
- * Initializes the feed list handling.
- */
-void feedlist_init (void);
+GType feedlist_get_type (void);
 
 /**
- * Destroys the feed list structure.
- */
-void feedlist_free (void);
-
-/**
- * Get feed list root node.
+ * Set up the feed list.
  *
- * @returns root node
+ * @returns the feed list instance
  */
-nodePtr feedlist_get_root (void);
+FeedList * feedlist_create (void);
 
 /**
  * Get currently selected feed list node 
@@ -73,18 +68,34 @@ nodePtr feedlist_get_root (void);
 nodePtr feedlist_get_selected (void);
 
 /**
- * Get the node insertion point for new subscriptions. 
+ * Query overall number of of unread items.
  *
- * @returns a parent node for new subscriptions
+ * @returns overall number of unread items.
  */
-nodePtr feedlist_get_insertion_point (void);
-
-/** statistic counter handling methods */
 guint feedlist_get_unread_item_count (void);
+
+/**
+ * Query overall number of new items.
+ *
+ * Note: result might be slightly off, but error
+ * won't aggregate over time.
+ *
+ * @returns overall number of new items.
+ */
 guint feedlist_get_new_item_count (void);
 
+/**
+ * Changes the overall number of new items.
+ *
+ * Note: Do not use this method directly, use 
+ * feedlist_node_was_updated() instead.
+ */
+void feedlist_update_new_item_count (guint addValue);
+
 /**
  * Reset the global feed list new item counter.
+ *
+ * @todo: use signal instead
  */
 void feedlist_reset_new_item_count (void);
 
@@ -95,6 +106,8 @@ void feedlist_reset_new_item_count (void);
  *
  * @param node		the updated node
  * @param newCount	number of new and unread items
+ *
+ * @todo: use signal instead
  */
 void feedlist_node_was_updated (nodePtr node, guint newCount);
 
@@ -103,26 +116,77 @@ void feedlist_node_was_updated (nodePtr node, guint newCount);
  *
  * @returns the feed list root node
  */
-nodePtr feedlist_get_root(void);
+nodePtr feedlist_get_root (void);
 
 /**
- * Removes the given node from the feed list and 
- * triggers the permanent removal of the node from cache.
+ * Adds a new subscription to the feed list.
+ *
+ * @param source	the subscriptions source URL
+ * @param filter	NULL or the filter for the subscription
+ * @param options	NULL or the update options
+ * @param flags		download request flags
+ */
+void feedlist_add_subscription (const gchar *source, const gchar *filter, updateOptionsPtr options, gint flags);
+
+/**
+ * Adds a folder to the feed list without any user interaction.
+ *
+ * @param title		the title of the new folder.
+ */
+void feedlist_add_folder (const gchar *title);
+
+/**
+ * Notifies the feed list controller that a new node
+ * was added to the feed list. This method will insert
+ * the new node into the feed list view and select 
+ * the new node.
+ *
+ * This method is used for all node types (feeds, folders...).
+ * 
+ * Before calling this method the node must be given
+ * a parent node using node_set_parent().
+ *
+ * @param node		the new node
+ */
+void feedlist_node_added (nodePtr node);
+
+/**
+ * Notifies the feed list controller that a new node
+ * was added to the feed list. Similar to feedlist_node_added()
+ * the new node will be added to the feed list but the
+ * selection won't be changed.
+ *
+ * This method is used for all node types (feeds, folders...).
+ * 
+ * Before calling this method the node must be given
+ * a parent node using node_set_parent().
+ *
+ * @param node		the new node
+ */
+void feedlist_node_imported (nodePtr node);
+
+/**
+ * Removes the given node from the feed list.
  *
  * @param node		the node to remove
  */
-void feedlist_remove_node(nodePtr node);
+void feedlist_remove_node (nodePtr node);
 
 /**
- * Synchronously saves the feed list. Only to be used upon exit!
+ * Notifies the feed list controller that an existing
+ * node was removed from it's source (feed list subtree)
+ * and is to be destroyed and to be removed from the 
+ * feed list view.
+ *
+ * @param node		the removed node
  */
-void feedlist_save(void);
+void feedlist_node_removed (nodePtr node);
 
 /**
  * Schedules a save requests for the feed list within the next 5s.
  * Triggers state saving for all feed list sources.
  */
-void feedlist_schedule_save(void);
+void feedlist_schedule_save (void);
 
 /**
  * Resets the update counter of all childs of the given node
@@ -131,6 +195,8 @@ void feedlist_schedule_save(void);
  */
 void feedlist_reset_update_counters (nodePtr node);
 
+gboolean feedlist_is_writable (void);
+
 /**
  * Triggers a recursive mark-all-read on the given node
  * and updates the feed list afterwards.
@@ -167,7 +233,7 @@ void feedlist_mark_all_read (nodePtr node);
  *
  * @param node		the new selected node
  */
-void feedlist_selection_changed(nodePtr node);
+void feedlist_selection_changed (nodePtr node);
 
 /** 
  * Tries to find the first node with an unread item in the given folder.
@@ -176,15 +242,8 @@ void feedlist_selection_changed(nodePtr node);
  * 
  * @return a found node or NULL
  */
-nodePtr	feedlist_find_unread_feed(nodePtr folder);
+nodePtr	feedlist_find_unread_feed (nodePtr folder);
 
-/* menu callbacks */
-void on_menu_delete(GtkWidget *widget, gpointer user_data);
-
-void on_menu_update(GtkWidget *widget, gpointer user_data);
-void on_menu_update_all(GtkWidget *widget, gpointer user_data);
-
-void on_menu_allread(GtkWidget *widget, gpointer user_data);
-void on_menu_allfeedsread(GtkWidget *widget, gpointer user_data);
+G_END_DECLS
 
 #endif
