@@ -1,7 +1,7 @@
 /**
- * @file htmlview.c implementation of the item view interface for HTML rendering
+ * @file htmlview.c  item view interface for HTML rendering
  * 
- * Copyright (C) 2006-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2009 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,14 +34,12 @@
 #include "itemset.h"
 #include "render.h"
 #include "vfolder.h"
-#include "ui/ui_htmlview.h"
+#include "ui/liferea_htmlview.h"
 
 // FIXME: namespace clash of LifereaHtmlView *htmlview and htmlView_priv 
 // clearly shows the need to merge htmlview.c and src/ui/ui_htmlview.c,
 // maybe with a separate a HTML cache object...
 
-extern htmlviewPluginPtr htmlviewPlugin;
-
 static struct htmlView_priv 
 {
 	GHashTable	*chunkHash;	/**< cache of HTML chunks of all displayed items */
@@ -143,7 +141,7 @@ htmlview_remove_item (itemPtr item)
 void
 htmlview_select_item (itemPtr item) 
 {
-	debug1(DEBUG_HTML, "HTML view: selecting \"%s\"", item_get_title(item));	
+	debug1(DEBUG_HTML, "HTML view: selecting \"%s\"", item?item_get_title(item):"none");
 	/* nothing to do... */
 }
 
@@ -188,12 +186,13 @@ itemset_to_xml (nodePtr node)
 	xmlNewTextChild (itemSetNode, NULL, "favicon", node_get_favicon_file (node));
 	xmlNewTextChild (itemSetNode, NULL, "title", node_get_title (node));
 
-	if (node->subscription && subscription_get_source (node->subscription))
-	       xmlNewTextChild (itemSetNode, NULL, "source", subscription_get_source (node->subscription));
-
-	// FIXME: should not be node type specific! Can be fixed by moving the feed link into subscription metadata!
-	if (IS_FEED (node))
-	       xmlNewTextChild (itemSetNode, NULL, "link", feed_get_html_url (node->data));
+	if (node->subscription) {
+		if (subscription_get_source (node->subscription))
+			xmlNewTextChild (itemSetNode, NULL, "source", subscription_get_source (node->subscription));
+			
+		if (subscription_get_homepage (node->subscription))
+			xmlNewTextChild (itemSetNode, NULL, "link", subscription_get_homepage (node->subscription));
+	}
 
 	return doc;
 }
@@ -231,7 +230,7 @@ htmlview_render_item (itemPtr item,
 	render_parameter_add (params, "pixmapsDir='file://" PACKAGE_DATA_DIR G_DIR_SEPARATOR_S PACKAGE G_DIR_SEPARATOR_S "pixmaps" G_DIR_SEPARATOR_S "'");
 	render_parameter_add (params, "baseUrl='%s'", baseUrl);
 	render_parameter_add (params, "summary='%d'", summaryMode?1:0);
-	render_parameter_add (params, "single='%d'", (viewMode == ITEMVIEW_SINGLE_ITEM)?0:1);
+	render_parameter_add (params, "single='%d'", (viewMode == ITEMVIEW_SINGLE_ITEM)?1:0);
 	output = render_xml (doc, "item", params);
 	
 	/* For debugging use: xmlSaveFormatFile("/tmp/test.xml", doc, 1); */
@@ -243,6 +242,8 @@ htmlview_render_item (itemPtr item,
 	return output;
 }
 
+extern htmlviewImplPtr htmlview_get_impl();
+
 void 
 htmlview_start_output (GString *buffer,
                        const gchar *base,
@@ -265,7 +266,7 @@ htmlview_start_output (GString *buffer,
 	}
 
 	if (css)
-		g_string_append (buffer, render_get_css (htmlviewPlugin->externalCss));
+		g_string_append (buffer, render_get_css (TRUE /* external CSS supported */));
 	
 	/* add predefined scripts to be used for item menu */
 	if (script) 
@@ -302,17 +303,16 @@ htmlview_start_output (GString *buffer,
 		""
 		"	window.clearTimeout(popupTimeout);"
 		"}"
-		""
 		"</script>");
 	}
 	
-	g_string_append (buffer, "</head>\n<body>");
+	g_string_append (buffer, "</head>");
 }
 
 void
 htmlview_finish_output (GString *buffer) 
 {
-	g_string_append (buffer, "</body></html>"); 
+	g_string_append (buffer, "</html>"); 
 }
 
 void
@@ -324,22 +324,18 @@ htmlview_update (LifereaHtmlView *htmlview, guint mode)
 	gchar		*baseURL = NULL;
 	gboolean	summaryMode;
 		
-	if (!htmlView_priv.node)
-	{
-		debug0 (DEBUG_HTML, "clearing HTML view as nothing is selected");
-		liferea_htmlview_clear (htmlview);
-		return;
-	}
-	
 	/* determine base URL */
 	switch (mode) {
 		case ITEMVIEW_SINGLE_ITEM:
 			item = itemlist_get_selected ();
-			if(item)
-				baseURL = (gchar *)node_get_base_url (node_from_id(item->nodeId));
+			if(item) {
+				baseURL = (gchar *)node_get_base_url (node_from_id (item->nodeId));
+				item_unload (item);
+			}
 			break;
 		default:
-			baseURL = (gchar *) node_get_base_url (htmlView_priv.node);
+			if (htmlView_priv.node)
+				baseURL = (gchar *) node_get_base_url (htmlView_priv.node);
 			break;
 	}
 
@@ -361,6 +357,8 @@ htmlview_update (LifereaHtmlView *htmlview, guint mode)
 					g_string_append (output, html);
 					g_free (html);
 				}
+				
+				item_unload (item);
 			}
 			break;
 		case ITEMVIEW_ALL_ITEMS:
@@ -370,7 +368,8 @@ htmlview_update (LifereaHtmlView *htmlview, guint mode)
 			   sets displaying everything in summary because of only a
 			   single feed without item descriptions would make no sense. */
 
-			summaryMode = !IS_FOLDER (htmlView_priv.node) && 
+			summaryMode = (NULL != htmlView_priv.node) &&
+			              !IS_FOLDER (htmlView_priv.node) && 
 	        		      !IS_VFOLDER (htmlView_priv.node) && 
 	        		      (htmlView_priv.missingContent > 3);
 
@@ -398,10 +397,14 @@ htmlview_update (LifereaHtmlView *htmlview, guint mode)
 			break;
 		case ITEMVIEW_NODE_INFO:
 			{
-				gchar *html = node_render (htmlView_priv.node);
-				if (html) {
-					g_string_append (output, html);
-					g_free (html);
+				gchar *html;
+				
+				if (htmlView_priv.node) {
+					html = node_render (htmlView_priv.node);	
+					if (html) {
+						g_string_append (output, html);
+						g_free (html);
+					}
 				}
 			}
 			break;
