@@ -1,7 +1,7 @@
 /**
  * @file ui_update.c GUI update monitor
  * 
- * Copyright (C) 2006-2007 Lars Lindner <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2008 Lars Lindner <lars.lindner@gmail.com>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,16 +18,15 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
  */
 
-#include <glib.h>
+#include "ui/ui_update.h" 
 
 #include "common.h"
 #include "feedlist.h"
 #include "node.h"
 #include "subscription.h"
 #include "update.h"
-#include "ui/ui_dialog.h"
+#include "ui/liferea_dialog.h"
 #include "ui/ui_node.h"
-#include "ui/ui_update.h" 
 
 enum {
 	UM_FAVICON,
@@ -111,9 +110,6 @@ ui_update_cancel (nodePtr node)
 	if (!node->subscription)
 		return;
 		
-	if (!node->subscription->updateJob)
-		return;
-		
 	subscription_cancel_update (node->subscription);
 }
 
@@ -180,7 +176,7 @@ void on_menu_show_update_monitor(GtkWidget *widget, gpointer user_data) {
 		/* Fill in data */
 		um1hash = g_hash_table_new(g_str_hash, g_str_equal);
 		um2hash = g_hash_table_new(g_str_hash, g_str_equal);
-	 	(void)g_timeout_add(1000, ui_update_monitor_update, NULL);
+	 	(void)g_timeout_add_seconds(1, ui_update_monitor_update, NULL);
 	}
 	
 	gtk_window_present(GTK_WINDOW(umdialog));
